//************************************************************************
//This module contains fitness case and functions to compute raw fitness *
//as well as standardized fitness for Symbolic Regression Problem        *
//Programmed by NXH.                                                     *
//************************************************************************
#ifndef __SYMBOLICREG__
#define __SYMBOLICREG__

#include<iostream.h>
#include<string.h>
#include<stdio.h>
#include<assert.h>
#include<math.h>
#include<stdlib.h>
#include "gconst.h"
#include "random.h"
#include "stree.h"
#include "grammar.h"

#define NUMFITCASE 20
#define INFPLUS  exp(700)
#define INFMINUS -exp(700)
//#define EXPOVL 650
#define PVAL(y) ((y==HUGE_VAL)?(INFPLUS):((y==-HUGE_VAL)?(-INFMINUS):(y)))
//****************************************
//Fitness cases for symbolic regression  *
//They are sampled between 0  and 1      *
//Seed for sampling 12345                *
//****************************************
struct _sample{
		 double x;
		 double y;
		};
typedef struct _sample sample;
sample fitcase[NUMFITCASE];
BOOL SuccPredicate=FALSE;
void SetFitCase()
{
 //FILE *fp;
  double x;
  int i;
//  char filename[50];
//  cout<<endl<<"Input Data File Name:";
//  cin>>filename;
//  fp=fopen(filename,"rt");assert(fp!=NULL);
  for(i=0;i<NUMFITCASE;i++)
   {
    // fscanf(fp,"%lf%lf",&x,&y);
     fitcase[i].x=2*Next_Double()-1;
     x=fitcase[i].x;
    // fitcase[i].y=x*x*x+x*x+x;
     fitcase[i].y=x*x*x*x+x*x*x+x*x+x;
     //fitcase[i].y=x*x*x*x*x+ x*x*x*x+ x*x*x + x*x + x;
      //fitcase[i].y=x*x*x*x*x*x + x*x*x*x*x + x*x*x*x + x*x*x + x*x + x;    
 }
 // fclose(fp);
}
//********************************************************************
//This function compute the function (in tree form) with given input *
//********************************************************************
double Compute(node *st,double x)
{
 node *p;
 double l,r;
 if (!strcmp(st->name,"X"))
     return x;
 else //st->name="EXP"
  {
    p=st->children;
    if (!strcmp(p->name,"EXP"))
    {
     l=Compute(p,x);//compute the left branch
     p=p->sibling;
     r=Compute(p->sibling,x);//compute the right branch
     p=p->children;//get the operator
     if (!strcmp(p->name,"add"))
       return PVAL(r+l);
     else
      if (!strcmp(p->name,"sub"))
	return PVAL(l-r);
	else
	 if (!strcmp(p->name,"mul"))
	  return PVAL(r*l);
	  else
	    if (r!=0)   //protected division
	     return PVAL(l/r);
	     else
	      return 1;
    }
   else
    if (!strcmp(p->name,"PRE"))
     {
       r=Compute(p->sibling,x); //compute exp inside function
       p=p->children;
       if (!strcmp(p->name,"sin"))
	return sin(r);
       else
	if(!strcmp(p->name,"cos"))
	 return cos(r);
	  else
	  if(!strcmp(p->name,"ep"))
	      return PVAL(exp(r));
	   else
	     if (r!=0)   //protected logarithm
	      return log(fabs(r));
	       else
		return 0;
     }
    else //VAR
      return Compute(p->children,x);
  }
}
//*********************************************
//Compute the raw fitness for an invidual     *
//*********************************************
double ComputeRF(individual st)
{
  double sum=0;
  double t;
  int i;
  int hit;
  hit=0;
  for(i=0;i<NUMFITCASE;i++)
   {
     t=PVAL(fabs(Compute(st.chrom,fitcase[i].x)-fitcase[i].y));
     if(t<=0.01)
      hit++;
      sum+=t;
   }
   if(hit==NUMFITCASE)
     SuccPredicate=TRUE;
   return sum;
}
//****************************************************************
//This function initializes elementary trees and adjoining table *
//****************************************************************
void LoadTerm()
{
 cout<<endl<<"Loading symbols and grammar rules...";
 //First Initialise Symbol  
 InitSymbol();
//Nonterminal symbols 
 AddSymbol("EXP",TRUE);
 AddSymbol("VAR",TRUE);
 AddSymbol("OP",TRUE);
 AddSymbol("PRE",TRUE);
//Terminal symbols 
 AddSymbol("add",FALSE);
 AddSymbol("mul",FALSE);
 AddSymbol("sub",FALSE);
 AddSymbol("div",FALSE);
 AddSymbol("sin",FALSE);
 AddSymbol("cos",FALSE);
 AddSymbol("ep",FALSE);
 AddSymbol("lg",FALSE);
 AddSymbol("X",FALSE);

//Second Initialise the rule set
 InitRuleSet();
 AddRule("EXP-> EXP OP EXP",1.0);
 AddRule("EXP-> PRE EXP",1.0);
 AddRule("EXP->  VAR",1.0);
 AddRule("OP-> add",1.0);
 AddRule("OP-> sub",1.0);
 AddRule("OP-> mul",1.0);
 AddRule("OP-> div",1.0);
 AddRule("PRE-> sin",1.0);
 AddRule("PRE-> cos",1.0);
 AddRule("PRE-> ep",1.0);
 AddRule("PRE-> lg",1.0);
 AddRule("VAR-> X",1.0);

//Finally Labeling the rules
 LabelRule();

 cout<<endl<<"Done!";
}
#endif
